﻿//!CompilerOption:AddRef:clipper_library.dll

using System.Collections.Generic;
using System.Threading.Tasks;
using Buddy.Wildstar.BotCommon;
using Buddy.Wildstar.Game;
using GoodStalker.Behaviours;
using GoodStalker.Controllers;
using GoodStalker.Dodge;
using Buddy.Coroutines;
using Buddy.Wildstar.Game.Actors;

namespace GoodStalker
{
    /// <summary>
    ///     GoodStalker - A simple Stalker routine designed to work with level 50 Stalkers.
    /// </summary>
    public class GoodStalker : CombatRoutineBase
    {
        private readonly BehaviourRunner _behaviourRunner;
        private readonly BehaviourRunner _combatBehaviourRunner;
        private readonly BehaviourRunner _pullBehaviourRunner;
        private readonly SpellController _spellController;

        public GoodStalker()
        {
            _spellController = new SpellController();
            _combatBehaviourRunner = new BehaviourRunner(new List<Behaviour>
            {
                new Survival(_spellController),
                new Combat(_spellController)
            });

            _behaviourRunner = new BehaviourRunner(new List<Behaviour>
            {
                new Rest(_spellController)
            });

            _pullBehaviourRunner = new BehaviourRunner(new List<Behaviour>
            {
                new Pull(_spellController)
            });

            Logger.Log("Initialised");
        }

        public override string Name
        {
            get { return Info.Name; }
        }

        public override string Author
        {
            get { return Info.Author; }
        }

        public override string Version
        {
            get { return Info.Version; }
        }

        public override async Task Combat()
        {
            if (GameManager.LocalPlayer.IsDead)
            {
                Evading.IsEvading = false;
                return;
            }

            await _combatBehaviourRunner.Execute();
        }

        public override async Task Pull(object preferredTarget)
        {
            // Safe-cast to Actor. (Null if not an actor, otherwise, an Actor object)
            Targeting.PullTarget = preferredTarget as Actor;

            if (Targeting.PullTarget == null)
            {
                Logger.Log("Non-Actor object passed into Pull method. Don't know what to do!");
                return;
            }

            // Ensure it's targeted
            if (GameManager.LocalPlayer.CurrentTargetGuid != Targeting.PullTarget.Guid)
            {
                Targeting.PullTarget.Target();
                await Coroutine.Sleep(100);
            }


            await _pullBehaviourRunner.Execute();
            await Combat();
        }

        public override void Pulse()
        {
            if (Evading.IsEvading && !GameManager.LocalPlayer.Position.IsInDanger() && GameManager.LocalPlayer.IsMoving)
            {
                Evading.IsEvading = false;
                Navigator.Stop();
                Logger.Log("Finished evading");
            }
            _behaviourRunner.Execute();
            base.Pulse();
        }
    }
}